/*
   SAPI32MC.H - SAPI32 User functions and data structures
   Copyright (c) 1996 Sangoma Technologies Inc.
   Author: Glenn Robins
   API v1.0, 32-bit, May 1996
  
   Application include file for SAPI32MC.DLL functions.
   See SAPI32.HLP for a complete function reference.

   Must #include <windows.h> before this header for type
   definitions used here.

   If compiling with Borland C, make sure structures are
   packed on byte boundaries. Link your application with
   SAPI32IM.DEF.

   If compiling with MS Visual C++ and for some reason
   you don't want to use __declspec(dllimport) then define
   SANGOMAIMPORT on the compiler command line. Link a VC++
   application using SAPI32MC.LIB.
*/

#ifndef SANGOMAAPI
#define SANGOMAAPI __cdecl
#endif

#ifndef SANGOMAVBASIC
#define SANGOMAVBASIC __stdcall
#endif

#ifdef SANGOMABUILD
#define SANGOMAIMPORT
#endif

#ifndef SANGOMAIMPORT
#if (_MSC_VER >= 800) && !defined(__STDC__)
#define SANGOMAIMPORT __declspec(dllimport)
#else
#define SANGOMAIMPORT
#endif
#endif

#ifndef SANGOMABUILD
#define SDLAR_NOT_REGISTERED    0L
#define SDLAR_DEREGISTERED              1L
#define SDLAR_RELEASED                  2L
#define SDLAR_REGISTERED        1L
#define SDLAR_ALLOCATED                 2L
#define SDLAR_CARD_BLOCKED              0L
#define SDLAR_PREPARED                  2L
#define SDLAR_NOT_PREPARED              0L
#define SDLAR_ERROR_PREPARING   3L
#endif


#define MDATALEN        544
#define CB_HEADERLEN    15

/* note: structure must be packed on 1-byte boundaries
   and for a block this size, it is not wise to allocate it on
   the stack - should be a static global
*/
/* control block */
#ifdef _MSC_VER
#pragma pack(1)
#endif
typedef struct {
	BYTE command;
	WORD buffer_length;
	BYTE return_code;
	BYTE sdlc_address;
	BYTE PF_bit;
	WORD poll_interval;
	BYTE general_mailbox_byte;
	BYTE reserved[6];
	BYTE data[MDATALEN];
} SDLC_BLOCK;
#ifdef _MSC_VER
#pragma pack()
#endif

#if defined (__cplusplus)
extern "C" {
#endif

/* SAPI will always return 1 */
SANGOMAIMPORT
int SANGOMAAPI SAPIAlive(void);

/* Initializes SAPI and loads SDLA driver, if not loaded already.
   A trace should be registered before this setup if one will
   be used at all, but is not necessary.
   Each application instance MUST call this on initialization.
   returns 0 if failed to load SDLA driver,
	   1 if this is the first setup call and the SDLA driver
	     loaded successfully,
	   2 if this is not the first successful setup call
	     (driver already loaded)
*/
SANGOMAIMPORT
int SANGOMAAPI SAPISetup(void);

/* Cleanup and unloads SDLA driver if this is the last
   application instance to use the driver.
   You must terminate your trace registration first if one
   was registered, unless you are the last application to use
   this DLL (but you shouldn't assume this).
   Each application instance MUST call this before closing.
   returns 1 always (return code reserved for future use)
*/
SANGOMAIMPORT
int SANGOMAAPI SAPICleanup(void);

/* Command call with control block
   returns return_code value in structure
*/
SANGOMAIMPORT
char SANGOMAAPI SAPIControl(SDLC_BLOCK *block, BYTE cardnumber);

/* returns 0 if a debugging command trace is already registered or
   had difficulty creating the window, 1 if successful.
   hWnd is a window to send notification messages to, and
   iMessage is a user-defined message that SAPI will send the
   application to signal an SAPI notification. The actual
   message code is passed by value to the WPARAM parameter
   of the window procedure
*/
SANGOMAIMPORT
char SANGOMAAPI SAPIRegisterDebug(HINSTANCE hInstance, HWND hWnd, UINT iMessage);

/* if a debugging command trace is active and instance matches that passed
   when registered (for security/fault tolerance purposes) then
   trace is terminated and returns 0. If instance doesn't match,
   returns 1. If no trace is running, returns 2.
   The trace is also terminated automatically if the window is
   closed (see trace messages)
*/
SANGOMAIMPORT
char SANGOMAAPI SAPITerminateDebug(HINSTANCE hInstance);

/* returns TRUE if a debugging command trace is registered and active */
SANGOMAIMPORT
BOOL SANGOMAAPI SAPIIsDebugRegistered(void);

/* DEBUGGING COMMAND TRACE MESSAGES (wParam values) */
#define TRACEMSG_CLOSE  0

/*
   returns SDLAR_NOT_REGISTERED (0L) if failed to obtain a selector
	(any read or write calls to the board will result
	in a general protection fault)
   returns SDLAR_REGISTERED if succeeded (1L): the card's
	instance count was incremented but this is not the
	first registration call for this board (it is already
	in use by an application)
   returns SDLAR_ALLOCATED if succeeded (2L): the card's
	instance count was incremented and this is the
	first registration call for this board, and the
	memory selector was allocated
*/
SANGOMAIMPORT
int SANGOMAAPI SAPIRegisterCard(BYTE cardnumber);

/*
   returns SDLAR_NOT_REGISTERED (0L) if card wasn't registered
   returns SDLAR_DEREGISTERED (1L) if card's instance count
	was decremented but isn't currently zero
	(i.e. an application is still using the specified board)
   returns SDLAR_RELEASED (2L) if card is now not used by
	any application, and the selector was freed
*/
SANGOMAIMPORT
int SANGOMAAPI SAPIDeregisterCard(BYTE cardnumber);

/*
   returns a bit record (32-bit) of cards registered for use
	and cards that exist:
	[ E15 E14 ... E3 E2 E1 E0 R15 R14 ... R3 R2 R1 R0 ]
   For example, if bit R2 is set, then card 2 is registered
		if bit E2 is set, then card 2 exists
			(i.e. a configuration entry exists
			in WIN.INI)
*/
SANGOMAIMPORT
LRESULT SANGOMAAPI SAPIGetCards(void);

/*
   reads a block of memory from the card's shared memory area
*/
SANGOMAIMPORT
void SANGOMAAPI SAPIReadSharedMemory(BYTE cardnumber, BYTE *p, WORD offset, WORD length);


/* Undocumented DLL exports - use at own risk */
SANGOMAIMPORT int SANGOMAAPI SAPIPrepareForLoading(BYTE cardnumber);
SANGOMAIMPORT int SANGOMAAPI SAPILoadComplete(BYTE cardnumber);
SANGOMAIMPORT unsigned char SANGOMAAPI SAPIReadPortByte(BYTE cardnumber, WORD offset);
SANGOMAIMPORT void SANGOMAAPI SAPIWritePortByte(BYTE cardnumber, WORD offset, BYTE data);
SANGOMAIMPORT unsigned char SANGOMAAPI SAPIReadByte(BYTE cardnumber, ULONG offset);
SANGOMAIMPORT void SANGOMAAPI SAPIWriteByte(BYTE cardnumber, ULONG offset, BYTE data);
SANGOMAIMPORT void SANGOMAAPI SAPIZeroWindow(BYTE cardnumber);
SANGOMAIMPORT void SANGOMAAPI SAPIStartS514(BYTE cardnumber);
SANGOMAIMPORT void SANGOMAAPI SAPIStopS514(BYTE cardnumber);

/* Exported function wrappers for Visual Basic people needing
   to call using _stdcall. Note that the functions without parameters
   are also defined with _stdcall just to be consistent.
   They are defined for internal use when building the DLL.
*/
#ifdef SANGOMABUILD
int SANGOMAVBASIC SAPIAliveStdCall(void);
int SANGOMAVBASIC SAPISetupStdCall(void);
int SANGOMAVBASIC SAPICleanupStdCall(void);
char SANGOMAVBASIC SAPIControlStdCall(SDLC_BLOCK *block, BYTE cardnumber);
char SANGOMAVBASIC SAPIRegisterDebugStdCall(HINSTANCE hInstance, HWND hWnd, UINT iMessage);
char SANGOMAVBASIC SAPITerminateDebugStdCall(HINSTANCE hInstance);
BOOL SANGOMAVBASIC SAPIIsDebugRegisteredStdCall(void);
int SANGOMAVBASIC SAPIRegisterCardStdCall(BYTE cardnumber);
int SANGOMAVBASIC SAPIDeregisterCardStdCall(BYTE cardnumber);
LRESULT SANGOMAVBASIC SAPIGetCardsStdCall(void);
void SANGOMAVBASIC SAPIReadSharedMemoryStdCall(BYTE cardnumber, BYTE *p, WORD offset, WORD length);
#endif

#ifdef __cplusplus
}
#endif
